@@ -149,6 +149,8 @@ public:
     AbstractMetaType* translateType(double vr, const AddedFunction::TypeInfo& typeInfo);
     AbstractMetaType *translateType(const TypeInfo &type, bool *ok, bool resolveType = true, bool resolveScope = true);
 
+    int findOutValueFromString(const QString& stringValue, bool& ok);
+
     void decideUsagePattern(AbstractMetaType *type);
 
     const AbstractMetaFunctionList globalFunctions() const
@@ -232,6 +234,7 @@ private:
     AbstractMetaArgumentList reverseList(const AbstractMetaArgumentList& list);
     void setInclude(TypeEntry* te, const QString& fileName) const;
     void fixArgumentNames(AbstractMetaFunction* func);
+    void fillAddedFunctions(AbstractMetaClass* metaClass);
 
     AbstractMetaClassList m_metaClasses;
     AbstractMetaClassList m_templates;
