@@ -325,7 +325,9 @@ public:
     }
     QString name() const
     {
-        return QString(m_typeEntry->targetLangName()).split("::").last();
+        if (m_name.isNull())
+            m_name = m_typeEntry->targetLangName().split("::").last();
+        return m_name;
     }
     QString fullName() const
     {
@@ -591,6 +593,8 @@ private:
     const TypeEntry *m_typeEntry;
     AbstractMetaTypeList m_instantiations;
     QString m_package;
+    mutable QString m_name;
+    mutable QString m_cachedCppSignature;
     QString m_originalTypeDescription;
 
     int m_arrayElementCount;
@@ -744,6 +748,8 @@ public:
 
     FieldModificationList modifications() const;
 
+    bool isModifiedRemoved(int types = TypeSystem::All) const;
+
     using AbstractMetaVariable::setDocumentation;
     using AbstractMetaVariable::documentation;
 
@@ -797,7 +803,8 @@ public:
             m_invalid(false),
             m_reverse(false),
             m_userAdded(false),
-            m_explicit(false)
+            m_explicit(false),
+            m_pointerOperator(false)
     {
     }
 
@@ -833,6 +840,20 @@ public:
         return m_reverse;
     }
 
+    /**
+     *  Returns true if this is a operator and the "self" operand is a pointer.
+     *  e.g. class Foo {}; operator+(SomeEnum, Foo*);
+     */
+    bool isPointerOperator() const
+    {
+        return m_pointerOperator;
+    }
+
+    void setPointerOperator(bool value)
+    {
+        m_pointerOperator = value;
+    }
+
     void setExplicit(bool isExplicit)
     {
         m_explicit = isExplicit;
@@ -1143,6 +1164,7 @@ private:
     QString m_name;
     QString m_originalName;
     mutable QString m_cachedMinimalSignature;
+    mutable QString m_cachedSignature;
     mutable QString m_cachedModifiedName;
 
     FunctionTypeEntry* m_typeEntry;
@@ -1159,6 +1181,7 @@ private:
     uint m_reverse                  : 1;
     uint m_userAdded                : 1;
     uint m_explicit                 : 1;
+    uint m_pointerOperator          : 1;
 };
 
 
@@ -1303,6 +1326,11 @@ public:
         m_class = c;
     }
 
+    bool isAnonymous() const
+    {
+        return m_typeEntry->isAnonymous();
+    }
+
 private:
     AbstractMetaEnumValueList m_enumValues;
     EnumTypeEntry *m_typeEntry;
@@ -1377,13 +1405,13 @@ public:
               m_hasEqualsOperator(false),
               m_hasCloneOperator(false),
               m_isTypeAlias(false),
+              m_hasToStringCapability(false),
               m_enclosingClass(0),
               m_baseClass(0),
               m_templateBaseClass(0),
               m_extractedInterface(0),
               m_primaryInterfaceImplementor(0),
               m_typeEntry(0),
-              //m_qDebugStreamFunction(0)
               m_stream(false)
     {
     }
@@ -1712,17 +1740,6 @@ public:
     {
         return m_hasHashFunction;
     }
-#if 0
-    void setToStringCapability(FunctionModelItem fun)
-    {
-        m_qDebugStreamFunction = fun;
-    }
-
-    FunctionModelItem hasToStringCapability() const
-    {
-        return m_qDebugStreamFunction;
-    }
-#endif
     virtual bool hasDefaultToStringFunction() const;
 
     void setHasEqualsOperator(bool on)
@@ -1870,6 +1887,15 @@ public:
         return m_stream;
     }
 
+    void setToStringCapability(bool value)
+    {
+        m_hasToStringCapability = value;
+    }
+
+    bool hasToStringCapability() const
+    {
+        return m_hasToStringCapability;
+    }
 private:
     uint m_namespace : 1;
     uint m_qobject : 1;
@@ -1887,7 +1913,8 @@ private:
     uint m_hasEqualsOperator : 1;
     uint m_hasCloneOperator : 1;
     uint m_isTypeAlias : 1;
-    uint m_reserved : 18;
+    uint m_hasToStringCapability : 1;
+    uint m_reserved : 17;
 
     const AbstractMetaClass *m_enclosingClass;
     AbstractMetaClass *m_baseClass;
