@@ -38,6 +38,8 @@ Handler::Handler(TypeDatabase* database, bool generate)
 {
     m_currentEnum = 0;
     m_current = 0;
+    m_currentDroppedEntry = 0;
+    m_currentDroppedEntryDepth = 0;
 
     tagNames["rejection"] = StackElement::Rejection;
     tagNames["primitive-type"] = StackElement::PrimitiveTypeEntry;
@@ -123,6 +125,18 @@ void Handler::fetchAttributeValues(const QString &name, const QXmlAttributes &at
 
 bool Handler::endElement(const QString &, const QString &localName, const QString &)
 {
+    if (m_currentDroppedEntry) {
+        if (m_currentDroppedEntryDepth == 1) {
+            m_current = m_currentDroppedEntry->parent;
+            delete m_currentDroppedEntry;
+            m_currentDroppedEntry = 0;
+            m_currentDroppedEntryDepth = 0;
+        } else {
+            ++m_currentDroppedEntryDepth;
+        }
+        return true;
+    }
+
     QString tagName = localName.toLower();
     if (tagName == "import-file")
         return true;
@@ -131,14 +145,10 @@ bool Handler::endElement(const QString &, const QString &localName, const QStrin
         return true;
 
     switch (m_current->type) {
-    case StackElement::AddFunction:
-        if (m_generate == TypeEntry::GenerateAll
-            && m_current->parent
-            && m_current->parent->type == StackElement::Root) { // Global function
-            TypeDatabase::instance()->addGlobalUserFunctions(m_addedFunctions);
-            TypeDatabase::instance()->addGlobalUserFunctionModifications(m_functionMods);
-            m_addedFunctions.clear();
-            m_functionMods.clear();
+    case StackElement::Root:
+        if (m_generate == TypeEntry::GenerateAll) {
+            TypeDatabase::instance()->addGlobalUserFunctions(m_contextStack.top()->addedFunctions);
+            TypeDatabase::instance()->addGlobalUserFunctionModifications(m_contextStack.top()->functionMods);
         }
         break;
     case StackElement::ObjectTypeEntry:
@@ -146,21 +156,16 @@ bool Handler::endElement(const QString &, const QString &localName, const QStrin
     case StackElement::InterfaceTypeEntry:
     case StackElement::NamespaceTypeEntry: {
         ComplexTypeEntry *centry = static_cast<ComplexTypeEntry *>(m_current->entry);
-        centry->setAddedFunctions(m_addedFunctions);
-        centry->setFunctionModifications(m_functionMods);
-        centry->setFieldModifications(m_fieldMods);
-        centry->setCodeSnips(m_codeSnips);
-        centry->setDocModification(m_docModifications);
+        centry->setAddedFunctions(m_contextStack.top()->addedFunctions);
+        centry->setFunctionModifications(m_contextStack.top()->functionMods);
+        centry->setFieldModifications(m_contextStack.top()->fieldMods);
+        centry->setCodeSnips(m_contextStack.top()->codeSnips);
+        centry->setDocModification(m_contextStack.top()->docModifications);
 
         if (centry->designatedInterface()) {
-            centry->designatedInterface()->setCodeSnips(m_codeSnips);
-            centry->designatedInterface()->setFunctionModifications(m_functionMods);
+            centry->designatedInterface()->setCodeSnips(m_contextStack.top()->codeSnips);
+            centry->designatedInterface()->setFunctionModifications(m_contextStack.top()->functionMods);
         }
-        m_codeSnips = CodeSnipList();
-        m_addedFunctions = AddedFunctionList();
-        m_functionMods = FunctionModificationList();
-        m_fieldMods = FieldModificationList();
-        m_docModifications = DocModificationList();
     }
     break;
     case StackElement::CustomMetaConstructor: {
@@ -174,8 +179,8 @@ bool Handler::endElement(const QString &, const QString &localName, const QStrin
     }
     break;
     case StackElement::EnumTypeEntry:
-        m_current->entry->setDocModification(m_docModifications);
-        m_docModifications = DocModificationList();
+        m_current->entry->setDocModification(m_contextStack.top()->docModifications);
+        m_contextStack.top()->docModifications = DocModificationList();
         m_currentEnum = 0;
         break;
     case StackElement::Template:
@@ -183,22 +188,31 @@ bool Handler::endElement(const QString &, const QString &localName, const QStrin
         break;
     case StackElement::TemplateInstanceEnum:
         if (m_current->parent->type == StackElement::InjectCode)
-            m_codeSnips.last().addTemplateInstance(m_current->value.templateInstance);
+            m_contextStack.top()->codeSnips.last().addTemplateInstance(m_current->value.templateInstance);
          else if (m_current->parent->type == StackElement::Template)
             m_current->parent->value.templateEntry->addTemplateInstance(m_current->value.templateInstance);
          else if (m_current->parent->type == StackElement::CustomMetaConstructor
                   || m_current->parent->type == StackElement::CustomMetaConstructor)
             m_current->parent->value.customFunction->addTemplateInstance(m_current->value.templateInstance);
          else if (m_current->parent->type == StackElement::ConversionRule)
-            m_functionMods.last().argument_mods.last().conversion_rules.last().addTemplateInstance(m_current->value.templateInstance);
+            m_contextStack.top()->functionMods.last().argument_mods.last().conversion_rules.last().addTemplateInstance(m_current->value.templateInstance);
          else if (m_current->parent->type == StackElement::InjectCodeInFunction)
-            m_functionMods.last().snips.last().addTemplateInstance(m_current->value.templateInstance);
+            m_contextStack.top()->functionMods.last().snips.last().addTemplateInstance(m_current->value.templateInstance);
 
         break;
     default:
         break;
     }
 
+    if (m_current->type == StackElement::Root
+        || m_current->type == StackElement::NamespaceTypeEntry
+        || m_current->type == StackElement::InterfaceTypeEntry
+        || m_current->type == StackElement::ObjectTypeEntry
+        || m_current->type == StackElement::ValueTypeEntry) {
+        StackElementContext* context = m_contextStack.pop();
+        delete context;
+    }
+
     StackElement *child = m_current;
     m_current = m_current->parent;
     delete(child);
@@ -208,6 +222,9 @@ bool Handler::endElement(const QString &, const QString &localName, const QStrin
 
 bool Handler::characters(const QString &ch)
 {
+    if (m_currentDroppedEntry)
+        return true;
+
     if (m_current->type == StackElement::Template) {
         m_current->value.templateEntry->addCode(ch);
         return true;
@@ -220,7 +237,7 @@ bool Handler::characters(const QString &ch)
 
     if (m_current->type == StackElement::ConversionRule
         && m_current->parent->type == StackElement::ModifyArgument) {
-        m_functionMods.last().argument_mods.last().conversion_rules.last().addCode(ch);
+        m_contextStack.top()->functionMods.last().argument_mods.last().conversion_rules.last().addCode(ch);
         return true;
     }
 
@@ -235,14 +252,14 @@ bool Handler::characters(const QString &ch)
                 break;
             case StackElement::ModifyFunction:
             case StackElement::AddFunction:
-                m_functionMods.last().snips.last().addCode(ch);
-                m_functionMods.last().modifiers |= FunctionModification::CodeInjection;
+                m_contextStack.top()->functionMods.last().snips.last().addCode(ch);
+                m_contextStack.top()->functionMods.last().modifiers |= FunctionModification::CodeInjection;
                 break;
             case StackElement::NamespaceTypeEntry:
             case StackElement::ObjectTypeEntry:
             case StackElement::ValueTypeEntry:
             case StackElement::InterfaceTypeEntry:
-                m_codeSnips.last().addCode(ch);
+                m_contextStack.top()->codeSnips.last().addCode(ch);
                 break;
             default:
                 Q_ASSERT(false);
@@ -252,7 +269,7 @@ bool Handler::characters(const QString &ch)
     }
 
     if (m_current->type & StackElement::DocumentationMask)
-        m_docModifications.last().setCode(ch);
+        m_contextStack.top()->docModifications.last().setCode(ch);
 
     return true;
 }
@@ -329,6 +346,37 @@ bool Handler::convertBoolean(const QString &_value, const QString &attributeName
     }
 }
 
+static bool convertRemovalAttribute(const QString& removalAttribute, Modification& mod, QString& errorMsg)
+{
+    QString remove = removalAttribute.toLower();
+    if (!remove.isEmpty()) {
+        if (remove == QLatin1String("all")) {
+            mod.removal = TypeSystem::All;
+        } else if (remove == QLatin1String("target")) {
+            mod.removal = TypeSystem::TargetLangAndNativeCode;
+        } else {
+            errorMsg = QString::fromLatin1("Bad removal type '%1'").arg(remove);
+            return false;
+        }
+    }
+    return true;
+}
+
+static void getNamePrefixRecursive(StackElement* element, QStringList& names)
+{
+    if (!element->parent || !element->parent->entry)
+        return;
+    getNamePrefixRecursive(element->parent, names);
+    names << element->parent->entry->name();
+}
+
+static QString getNamePrefix(StackElement* element)
+{
+    QStringList names;
+    getNamePrefixRecursive(element, names);
+    return names.join(".");
+}
+
 bool Handler::startElement(const QString &, const QString &n,
                            const QString &, const QXmlAttributes &atts)
 {
@@ -336,21 +384,28 @@ bool Handler::startElement(const QString &, const QString &n,
     if (tagName == "import-file")
         return importFileElement(atts);
 
-
-    StackElement *element = new StackElement(m_current);
-
     if (!tagNames.contains(tagName)) {
         m_error = QString("Unknown tag name: '%1'").arg(tagName);
         return false;
     }
 
-    element->type = tagNames[tagName];
-    if (element->type & StackElement::TypeEntryMask) {
-        if (m_current->type != StackElement::Root) {
-            m_error = "Nested types not supported";
-            return false;
-        }
+    if (m_currentDroppedEntry) {
+        ++m_currentDroppedEntryDepth;
+        return true;
+    }
 
+    StackElement* element = new StackElement(m_current);
+    element->type = tagNames[tagName];
+
+    if (element->type == StackElement::Root
+        || element->type == StackElement::NamespaceTypeEntry
+        || element->type == StackElement::InterfaceTypeEntry
+        || element->type == StackElement::ObjectTypeEntry
+        || element->type == StackElement::ValueTypeEntry) {
+        m_contextStack.push(new StackElementContext());
+    }
+
+    if (element->type & StackElement::TypeEntryMask) {
         QHash<QString, QString> attributes;
         attributes["name"] = QString();
         attributes["since"] = QString("0");
@@ -372,6 +427,7 @@ bool Handler::startElement(const QString &, const QString &n,
             attributes["lower-bound"] = QString();
             attributes["force-integer"] = "no";
             attributes["extensible"] = "no";
+            attributes["identified-by-value"] = QString();
             break;
         case StackElement::ObjectTypeEntry:
         case StackElement::ValueTypeEntry:
@@ -399,6 +455,7 @@ bool Handler::startElement(const QString &, const QString &n,
             break;
         case StackElement::FunctionTypeEntry:
             attributes["signature"] = QString();
+            attributes["rename"] = QString();
             break;
         default:
             { } // nada
@@ -408,11 +465,35 @@ bool Handler::startElement(const QString &, const QString &n,
         QString name = attributes["name"];
         double since = attributes["since"].toDouble();
 
+        if (m_database->hasDroppedTypeEntries()) {
+            QString identifier = getNamePrefix(element) + '.';
+            identifier += (element->type == StackElement::FunctionTypeEntry ? attributes["signature"] : name);
+            if (m_database->shouldDropTypeEntry(identifier)) {
+                m_currentDroppedEntry = element;
+                m_currentDroppedEntryDepth = 1;
+                ReportHandler::debugSparse(QString("Type system entry '%1' was intentionally dropped from generation.").arg(identifier));
+                return true;
+            }
+        }
+
         // The top level tag 'function' has only the 'signature' tag
         // and we should extract the 'name' value from it.
         if (element->type == StackElement::FunctionTypeEntry) {
             QString signature = attributes["signature"];
             name = signature.left(signature.indexOf('(')).trimmed();
+            QString rename = attributes["rename"];
+            if (!rename.isEmpty()) {
+                static QRegExp functionNameRegExp("^[a-zA-Z_][a-zA-Z0-9_]*$");
+                if (!functionNameRegExp.exactMatch(rename)) {
+                    m_error = "can not rename '" + signature + "', '" + rename + "' is not a valid function name";
+                    return false;
+                }
+                FunctionModification mod(since);
+                mod.signature = signature;
+                mod.renamedToName = attributes["rename"];
+                mod.modifiers |= Modification::Rename;
+                m_contextStack.top()->functionMods << mod;
+            }
         }
 
         // We need to be able to have duplicate primitive type entries,
@@ -425,6 +506,22 @@ bool Handler::startElement(const QString &, const QString &n,
                 ReportHandler::warning(QString("Duplicate type entry: '%1'").arg(name));
         }
 
+        if (element->type == StackElement::EnumTypeEntry) {
+            if (name.isEmpty()) {
+                name = attributes["identified-by-value"];
+            } else if (!attributes["identified-by-value"].isEmpty()) {
+                m_error = "can't specify both 'name' and 'identified-by-value' attributes";
+                return false;
+            }
+        }
+
+        // Fix type entry name using nesting information.
+        if (element->type & StackElement::TypeEntryMask
+            && element->parent && element->parent->type != StackElement::Root) {
+            name = element->parent->entry->name() + "::" + name;
+        }
+
+
         if (name.isEmpty()) {
             m_error = "no 'name' attribute specified";
             return false;
@@ -478,13 +575,13 @@ bool Handler::startElement(const QString &, const QString &n,
             break;
         case StackElement::EnumTypeEntry: {
             QStringList names = name.split(QLatin1String("::"));
-
             if (names.size() == 1)
                 m_currentEnum = new EnumTypeEntry(QString(), name, since);
              else
                 m_currentEnum =
                     new EnumTypeEntry(QStringList(names.mid(0, names.size() - 1)).join("::"),
                                       names.last(), since);
+            m_currentEnum->setAnonymous(!attributes["identified-by-value"].isEmpty());
             element->entry = m_currentEnum;
             m_currentEnum->setCodeGeneration(m_generate);
             m_currentEnum->setTargetLangPackage(m_defaultPackage);
@@ -688,7 +785,7 @@ bool Handler::startElement(const QString &, const QString &n,
             QString signature = m_current->type & StackElement::TypeEntryMask ? QString() : m_currentSignature;
             DocModification mod(mode, signature, since);
             mod.format = lang;
-            m_docModifications << mod;
+            m_contextStack.top()->docModifications << mod;
         } else {
             m_error = "inject-documentation must be inside modify-function, "
                       "modify-field or other tags that creates a type";
@@ -707,7 +804,7 @@ bool Handler::startElement(const QString &, const QString &n,
                                 | StackElement::ModifyField;
         if (m_current->parent && m_current->parent->type & validParent) {
             QString signature = (m_current->type & StackElement::TypeEntryMask) ? QString() : m_currentSignature;
-            m_docModifications << DocModification(attributes["xpath"], signature, since);
+            m_contextStack.top()->docModifications << DocModification(attributes["xpath"], signature, since);
         } else {
             m_error = "modify-documentation must be inside modify-function, "
                       "modify-field or other tags that creates a type";
@@ -781,6 +878,7 @@ bool Handler::startElement(const QString &, const QString &n,
             attributes["name"] = QString();
             attributes["write"] = "true";
             attributes["read"] = "true";
+            attributes["remove"] = QString();
             break;
         case StackElement::Access:
             attributes["modifier"] = QString();
@@ -911,7 +1009,7 @@ bool Handler::startElement(const QString &, const QString &n,
                 return false;
             }
 
-            m_functionMods.last().argument_mods.last().modified_type = attributes["modified-type"];
+            m_contextStack.top()->functionMods.last().argument_mods.last().modified_type = attributes["modified-type"];
         }
         break;
         case StackElement::ConversionRule: {
@@ -925,23 +1023,24 @@ bool Handler::startElement(const QString &, const QString &n,
                 return false;
             }
 
-            if (topElement.type == StackElement::ModifyArgument) {
-                static QHash<QString, TypeSystem::Language> languageNames;
-                if (languageNames.isEmpty()) {
-                    languageNames["target"] = TypeSystem::TargetLangCode;
-                    languageNames["native"] = TypeSystem::NativeCode;
-                }
+            static QHash<QString, TypeSystem::Language> languageNames;
+            if (languageNames.isEmpty()) {
+                languageNames["target"] = TypeSystem::TargetLangCode;
+                languageNames["native"] = TypeSystem::NativeCode;
+            }
 
-                QString languageAttribute = attributes["class"].toLower();
-                TypeSystem::Language lang = languageNames.value(languageAttribute, TypeSystem::NoLanguage);
-                if (lang == TypeSystem::NoLanguage) {
+            QString languageAttribute = attributes["class"].toLower();
+            TypeSystem::Language lang = languageNames.value(languageAttribute, TypeSystem::NoLanguage);
+
+            if (topElement.type == StackElement::ModifyArgument) {
+               if (lang == TypeSystem::NoLanguage) {
                     m_error = QString("unsupported class attribute: '%1'").arg(lang);
                     return false;
                 }
 
                 CodeSnip snip(since);
                 snip.language = lang;
-                m_functionMods.last().argument_mods.last().conversion_rules.append(snip);
+                m_contextStack.top()->functionMods.last().argument_mods.last().conversion_rules.append(snip);
             } else {
                 if (topElement.entry->hasConversionRule()) {
                     m_error = "Types can have only one conversion rule";
@@ -958,13 +1057,18 @@ bool Handler::startElement(const QString &, const QString &n,
                 //Handler constructor....
                 if (m_generate != TypeEntry::GenerateForSubclass
                     && m_generate != TypeEntry::GenerateNothing) {
+
+                    const char* conversionFlag = NATIVE_CONVERSION_RULE_FLAG;
+                    if (lang == TypeSystem::TargetLangCode)
+                        conversionFlag = TARGET_CONVERSION_RULE_FLAG;
+
                     QFile conversionSource(sourceFile);
                     if (conversionSource.open(QIODevice::ReadOnly | QIODevice::Text)) {
-                        topElement.entry->setConversionRule(QString::fromUtf8(conversionSource.readAll()));
+                        topElement.entry->setConversionRule(conversionFlag + QString::fromUtf8(conversionSource.readAll()));
                     } else {
                         ReportHandler::warning("File containing conversion code for "
                                                + topElement.entry->name()
-                                               + " type does not exist or is not redable: "
+                                               + " type does not exist or is not readable: "
                                                + sourceFile);
                     }
                 }
@@ -1005,7 +1109,7 @@ bool Handler::startElement(const QString &, const QString &n,
             ArgumentModification argumentModification = ArgumentModification(idx, since);
             argumentModification.replace_value = replace_value;
             argumentModification.resetAfterUse = convertBoolean(attributes["invalidate-after-use"], "invalidate-after-use", false);
-            m_functionMods.last().argument_mods.append(argumentModification);
+            m_contextStack.top()->functionMods.last().argument_mods.append(argumentModification);
         }
         break;
         case StackElement::NoNullPointers: {
@@ -1014,9 +1118,9 @@ bool Handler::startElement(const QString &, const QString &n,
                 return false;
             }
 
-            m_functionMods.last().argument_mods.last().noNullPointers = true;
-            if (!m_functionMods.last().argument_mods.last().index)
-                m_functionMods.last().argument_mods.last().nullPointerDefaultValue = attributes["default-value"];
+            m_contextStack.top()->functionMods.last().argument_mods.last().noNullPointers = true;
+            if (!m_contextStack.top()->functionMods.last().argument_mods.last().index)
+                m_contextStack.top()->functionMods.last().argument_mods.last().nullPointerDefaultValue = attributes["default-value"];
              else if (!attributes["default-value"].isEmpty())
                 ReportHandler::warning("default values for null pointer guards are only effective for return values");
 
@@ -1031,7 +1135,7 @@ bool Handler::startElement(const QString &, const QString &n,
             static QHash<QString, TypeSystem::Language> languageNames;
             if (languageNames.isEmpty()) {
                 languageNames["target"] = TypeSystem::TargetLangCode;
-                languageNames["shell"] = TypeSystem::ShellCode;
+                languageNames["native"] = TypeSystem::NativeCode;
             }
 
             QString classAttribute = attributes["class"].toLower();
@@ -1055,7 +1159,7 @@ bool Handler::startElement(const QString &, const QString &n,
                 return false;
             }
 
-            m_functionMods.last().argument_mods.last().ownerships[lang] = owner;
+            m_contextStack.top()->functionMods.last().argument_mods.last().ownerships[lang] = owner;
         }
         break;
         case StackElement::SuppressedWarning:
@@ -1089,7 +1193,7 @@ bool Handler::startElement(const QString &, const QString &n,
 
 
             if (topElement.type == StackElement::InjectCodeInFunction)
-                m_functionMods.last().snips.last().argumentMap[pos] = meta_name;
+                m_contextStack.top()->functionMods.last().snips.last().argumentMap[pos] = meta_name;
              else {
                 ReportHandler::warning("Argument maps are only useful for injection of code "
                                        "into functions.");
@@ -1115,7 +1219,7 @@ bool Handler::startElement(const QString &, const QString &n,
                 return false;
             }
 
-            m_functionMods.last().removal = lang;
+            m_contextStack.top()->functionMods.last().removal = lang;
         }
         break;
         case StackElement::Rename:
@@ -1129,9 +1233,9 @@ bool Handler::startElement(const QString &, const QString &n,
 
             Modification *mod = 0;
             if (topElement.type == StackElement::ModifyFunction)
-                mod = &m_functionMods.last();
+                mod = &m_contextStack.top()->functionMods.last();
             else if (topElement.type == StackElement::ModifyField)
-                mod = &m_fieldMods.last();
+                mod = &m_contextStack.top()->fieldMods.last();
 
             QString modifier;
             if (element->type == StackElement::Rename) {
@@ -1147,7 +1251,7 @@ bool Handler::startElement(const QString &, const QString &n,
                 else if (topElement.type == StackElement::ModifyField)
                     mod->setRenamedTo(renamed_to);
                 else
-                    m_functionMods.last().argument_mods.last().renamed_to = renamed_to;
+                    m_contextStack.top()->functionMods.last().argument_mods.last().renamed_to = renamed_to;
             } else
                 modifier = attributes["modifier"].toLower();
 
@@ -1183,7 +1287,7 @@ bool Handler::startElement(const QString &, const QString &n,
                 return false;
             }
 
-            m_functionMods.last().argument_mods.last().removed = true;
+            m_contextStack.top()->functionMods.last().argument_mods.last().removed = true;
             break;
 
         case StackElement::ModifyField: {
@@ -1194,13 +1298,16 @@ bool Handler::startElement(const QString &, const QString &n,
             fm.name = name;
             fm.modifiers = 0;
 
+            if (!convertRemovalAttribute(attributes["remove"], fm, m_error))
+                return false;
+
             QString read = attributes["read"];
             QString write = attributes["write"];
 
             if (read == "true") fm.modifiers |= FieldModification::Readable;
             if (write == "true") fm.modifiers |= FieldModification::Writable;
 
-            m_fieldMods << fm;
+            m_contextStack.top()->fieldMods << fm;
         }
         break;
         case StackElement::AddFunction: {
@@ -1235,11 +1342,11 @@ bool Handler::startElement(const QString &, const QString &n,
                 }
             }
 
-            m_addedFunctions << func;
+            m_contextStack.top()->addedFunctions << func;
 
             FunctionModification mod(since);
             mod.signature = m_currentSignature;
-            m_functionMods << mod;
+            m_contextStack.top()->functionMods << mod;
         }
         break;
         case StackElement::ModifyFunction: {
@@ -1280,18 +1387,8 @@ bool Handler::startElement(const QString &, const QString &n,
             if (convertBoolean(attributes["deprecated"], "deprecated", false))
                 mod.modifiers |= Modification::Deprecated;
 
-
-            QString remove = attributes["remove"].toLower();
-            if (!remove.isEmpty()) {
-                if (remove == QLatin1String("all"))
-                    mod.removal = TypeSystem::All;
-                else if (remove == QLatin1String("target"))
-                    mod.removal = TypeSystem::TargetLangAndNativeCode;
-                else {
-                    m_error = QString::fromLatin1("Bad removal type '%1'").arg(remove);
-                    return false;
-                }
-            }
+            if (!convertRemovalAttribute(attributes["remove"], mod, m_error))
+                return false;
 
             QString rename = attributes["rename"];
             if (!rename.isEmpty()) {
@@ -1308,7 +1405,7 @@ bool Handler::startElement(const QString &, const QString &n,
 
             mod.modifiers |= (convertBoolean(attributes["virtual-slot"], "virtual-slot", false) ? Modification::VirtualSlot : 0);
 
-            m_functionMods << mod;
+            m_contextStack.top()->functionMods << mod;
         }
         break;
         case StackElement::ReplaceDefaultExpression:
@@ -1322,10 +1419,10 @@ bool Handler::startElement(const QString &, const QString &n,
                 return false;
             }
 
-            m_functionMods.last().argument_mods.last().replacedDefaultExpression = attributes["with"];
+            m_contextStack.top()->functionMods.last().argument_mods.last().replacedDefaultExpression = attributes["with"];
             break;
         case StackElement::RemoveDefaultExpression:
-            m_functionMods.last().argument_mods.last().removedDefaultExpression = true;
+            m_contextStack.top()->functionMods.last().argument_mods.last().removedDefaultExpression = true;
             break;
         case StackElement::CustomMetaConstructor:
         case StackElement::CustomMetaDestructor: {
@@ -1345,9 +1442,9 @@ bool Handler::startElement(const QString &, const QString &n,
             static QHash<QString, ReferenceCount::Action> actions;
             if (actions.isEmpty()) {
                 actions["add"] = ReferenceCount::Add;
-                //actions["add-all"] = ReferenceCount::AddAll;
+                actions["add-all"] = ReferenceCount::AddAll;
                 actions["remove"] = ReferenceCount::Remove;
-                //actions["set"] = ReferenceCount::Set;
+                actions["set"] = ReferenceCount::Set;
                 actions["ignore"] = ReferenceCount::Ignore;
             }
             rc.action = actions.value(attributes["action"].toLower(), ReferenceCount::Invalid);
@@ -1359,7 +1456,7 @@ bool Handler::startElement(const QString &, const QString &n,
                     m_error += " " + action;
             }
 
-            m_functionMods.last().argument_mods.last().referenceCounts.append(rc);
+            m_contextStack.top()->functionMods.last().argument_mods.last().referenceCounts.append(rc);
         }
         break;
 
@@ -1396,7 +1493,7 @@ bool Handler::startElement(const QString &, const QString &n,
                 return false;
             }
             ao.index = idx;
-            m_functionMods.last().argument_mods.last().owner = ao;
+            m_contextStack.top()->functionMods.last().argument_mods.last().owner = ao;
         }
         break;
 
@@ -1479,20 +1576,20 @@ bool Handler::startElement(const QString &, const QString &n,
             }
 
             if (topElement.type == StackElement::ModifyFunction || topElement.type == StackElement::AddFunction) {
-                FunctionModification mod = m_functionMods.last();
+                FunctionModification mod = m_contextStack.top()->functionMods.last();
                 if (snip.language == TypeSystem::ShellDeclaration) {
                     m_error = "no function implementation in shell declaration in which to inject code";
                     return false;
                 }
 
-                m_functionMods.last().snips << snip;
+                m_contextStack.top()->functionMods.last().snips << snip;
                 if (in_file)
-                    m_functionMods.last().modifiers |= FunctionModification::CodeInjection;
+                    m_contextStack.top()->functionMods.last().modifiers |= FunctionModification::CodeInjection;
                 element->type = StackElement::InjectCodeInFunction;
             } else if (topElement.type == StackElement::Root) {
                 element->entry->addCodeSnip(snip);
             } else if (topElement.type != StackElement::Root) {
-                m_codeSnips << snip;
+                m_contextStack.top()->codeSnips << snip;
             }
 
         }
@@ -1976,6 +2073,22 @@ QString ContainerTypeEntry::typeName() const
     }
 }
 
+static bool strLess(const char* a, const char* b)
+{
+    return ::strcmp(a, b) < 0;
+}
+
+bool TypeEntry::isCppPrimitive() const
+{
+    if (m_name.contains(' ') || m_type == VoidType)
+        return true;
+    // Keep this sorted!!
+    static const char* cppTypes[] = { "bool", "char", "double", "float", "int", "long", "short", "wchar_t"};
+    const int N = sizeof(cppTypes)/sizeof(char*);
+
+    const char** res = qBinaryFind(&cppTypes[0], &cppTypes[N], m_name.toAscii().constData(), strLess);
+    return res != &cppTypes[N];
+}
 
 /*
 static void injectCode(ComplexTypeEntry *e,
