@@ -32,6 +32,10 @@
 #include "apiextractormacros.h"
 #include "include.h"
 
+//Used to identify the conversion rule to avoid break API
+#define TARGET_CONVERSION_RULE_FLAG "0"
+#define NATIVE_CONVERSION_RULE_FLAG "1"
+
 class Indentor;
 
 class AbstractMetaType;
@@ -105,7 +109,13 @@ struct ArgumentOwner
         Add         = 0x01,
         Remove      = 0x02
     };
-    ArgumentOwner() : action(ArgumentOwner::Invalid), index(-2) {}
+    enum {
+        InvalidIndex = -2,
+        ThisIndex = -1,
+        ReturnIndex = 0,
+        FirstArgumentIndex = 1
+    };
+    ArgumentOwner() : action(ArgumentOwner::Invalid), index(ArgumentOwner::InvalidIndex) {}
 
     Action action;
     int index;
@@ -306,7 +316,7 @@ struct APIEXTRACTOR_API Modification
         VirtualSlot =          0x10000 | NonFinal
     };
 
-    Modification() : modifiers(0) { }
+    Modification() : modifiers(0), removal(TypeSystem::NoLanguage) { }
 
     bool isAccessModifier() const
     {
@@ -364,22 +374,24 @@ struct APIEXTRACTOR_API Modification
         return modifiers & Rename;
     }
 
+    bool isRemoveModifier() const
+    {
+        return removal != TypeSystem::NoLanguage;
+    }
+
     uint modifiers;
     QString renamedToName;
+    TypeSystem::Language removal;
 };
 
 struct APIEXTRACTOR_API FunctionModification: public Modification
 {
-    FunctionModification(double vr) : removal(TypeSystem::NoLanguage), m_thread(false), m_allowThread(false), m_version(vr) {}
+    FunctionModification(double vr) : m_thread(false), m_allowThread(false), m_version(vr) {}
 
     bool isCodeInjection() const
     {
         return modifiers & CodeInjection;
     }
-    bool isRemoveModifier() const
-    {
-        return removal != TypeSystem::NoLanguage;
-    }
     void setIsThread(bool flag)
     {
         m_thread = flag;
@@ -410,7 +422,6 @@ struct APIEXTRACTOR_API FunctionModification: public Modification
     QString signature;
     QString association;
     CodeSnipList snips;
-    TypeSystem::Language removal;
 
     QList<ArgumentModification> argument_mods;
 
@@ -913,6 +924,7 @@ public:
         m_include = inc;
     }
 
+    // Replace conversionRule arg to CodeSnip in future version
     /// Set the type convertion rule
     void setConversionRule(const QString& conversionRule)
     {
@@ -922,7 +934,8 @@ public:
     /// Returns the type convertion rule
     QString conversionRule() const
     {
-        return m_conversionRule;
+        //skip conversions flag
+        return m_conversionRule.mid(1);
     }
 
     /// Returns true if there are any conversiton rule for this type, false otherwise.
@@ -936,7 +949,17 @@ public:
         return m_version;
     }
 
+    bool hasNativeConversionRule() const
+    {
+        return m_conversionRule.startsWith(NATIVE_CONVERSION_RULE_FLAG);
+    }
 
+    bool hasTargetConversionRule() const
+    {
+        return m_conversionRule.startsWith(TARGET_CONVERSION_RULE_FLAG);
+    }
+
+    bool isCppPrimitive() const;
 private:
     QString m_name;
     Type m_type;
@@ -1265,6 +1288,15 @@ public:
         m_forceInteger = force;
     }
 
+    bool isAnonymous() const
+    {
+        return m_anonymous;
+    }
+    void setAnonymous(bool anonymous)
+    {
+        m_anonymous = anonymous;
+    }
+
 private:
     QString m_packageName;
     QString m_qualifier;
@@ -1280,6 +1312,7 @@ private:
 
     bool m_extensible;
     bool m_forceInteger;
+    bool m_anonymous;
 };
 
 class APIEXTRACTOR_API EnumValueTypeEntry : public TypeEntry
